//#########################################################################
// File: commands.c
//
// Serial commands input
//
//#########################################################################
// Last change: 30.11.2007
//#########################################################################
// holger.klabunde@t-online.de
// http://home.t-online.de/home/holger.klabunde/homepage.htm
//#########################################################################
// Compiler: AVR-GCC 3.2
//#########################################################################
// edit by Kingstener 2007/2012/2013
// *thanks to H.Klabunde & A.Fitzenreiter*
//#########################################################################
#include <string.h>
#include "commands.h"
#include "serial.h"
#include "dos.h"
#include "mydefs.h"
#include "printf.h"
#include "fat.h"
#include "kcfile.h"
#include "pport.h"
#include "data.h"
#include "sddrv.h"
#include "kcscf.h"

unsigned int EOB(void);
void listdirl(void);
void listdir(void);
void sendstring(unsigned int tab);
//########################################################
unsigned char WaitCommand(void)
//########################################################
{
	unsigned char c;
	char s[2];
	char line[32];
	char lin[14];
	unsigned char pos = 0;
	unsigned int i;
	size_t strcspn();
	s[1]=0;
	strcpy(line,"\0");

	do                 //Kommando Eingabe
	{
		c=getp();

		//SD-OS
		//if(c<0x08){ sdfile(c+0xF0); return NOCOMM;} // testweise alternativ zu F0..F5
		if(c>=0xF0){ sdfile(c); return NOCOMM;}

		//sonst
		if (((c> 0x60) && (c < 0x7b))) c &= ~(1<< 5);//shiftlock
		if(c=='^')c='~';//^ als Tilde für lange Dateinamen
		if(c>0x1F){ s[0]=c;strcat(line,s);}
		if(c==0x08 && strlen(line)>0) line[strlen(line)-1]=0; //<-
	}
	while(c!=0x0D); // bis Enter
	
	if(line[0]==0) return NOCOMM;
	if(strlen(line)==1) //short commands without parameters
	{
		if(line[0]=='H') return HELP;
		if(line[0]=='?') return HELP;
		if(line[0]=='I') return INFO;
		if(line[0]=='Q'){printf("BYE...\n"); ser_putc(0x03); return NOCOMM;}
		if(line[0]=='V'){printf("\n" KCSF_VERSION " @ Kingstener 2012\n*thanks to A.Fitzenreiter & H.Klabunde*\nmod. V.Pohlers 2024\n");return NOCOMM;}
		if(line[0]=='D'){listdir();return NOCOMM;}
	}


	if(strlen(line)==2) //short commands without parameters
	{
		if(strcmp(line,"WB")==0) { sendstring(14); return NOCOMM; } //WBasic
		if(strcmp(line,"LL")==0) {listdirl();return NOCOMM;}//listlong experimentell!
	}
	if(strncmp(line,"RN ",3)==0)//rename
	{
		for(i=2;i<30;i++) {if (line[i]==',')pos=i;};	//position vom "," suchen
		strncpy(lin,&line[pos+1],12);					//Teil nach komma kopieren
		for(i=pos;i<30;i++)line[i]=0;					//oberen Teil l�schen
		if (Rename(&line[3],lin)==F_OK){printf("\nOK");}else printf("\nERROR");
		return NOCOMM;
	}
	if(strcmp(line,"EOR")==0){return EORAM;}
	if(strcmp(line,"EOB")==0){printf("\nBASIC-Ende:%04x",EOB());return NOCOMM;}
	if(strncmp(line,"MD ",3)==0)//make dir
	{
		if (Mkdir(&line[3])==F_OK){printf("\nOK");}else printf("\nERROR");
		return NOCOMM;
	}
	if(strncmp(line,"RM ",3)==0)//remove
	{
		if(FindName(&line[3])==FULL_MATCH)
		{
			printf("\nREMOVE Y/N?>");
			ser_putc(0x80);
			if(Command()== 'Y')
			{
				Remove(&line[3]);printf("\nOK");
			};
		}
		else printf("\nNOT FOUND");
		return NOCOMM;
	}
	
	if(strncmp(line,"CD ",3)==0)//change directory
	{
		if (Chdir(&line[3])!=F_OK) {printf("\nNOT FOUND");}
		return NOCOMM;
	}
	if(strncmp(line,"L ",2)==0) {KCload(&line[2]); return NOCOMM;} // KCPRGLOAD;
	if(strncmp(line,"S ",2)==0) {KCsave(&line[2]); return NOCOMM;} //KCPRGSAVE;
	if(strncmp(line,"T ",2)==0) {KCsaveTAP(&line[2]); return NOCOMM;} //KCPRGSAVE;
	if(strncmp(line,"M ",2)==0) {memo(&line[2]); return NOCOMM;}//Speicher/File anzeigen
	if(strncmp(line,"J ",2)==0) {jump(&line[2]); return NOCOMM;}//Programm starten
	if(strncmp(line,"LR ",3)==0) {rawload(&line[3]); return NOCOMM;}//load raw
	if(strncmp(line,"SR ",3)==0) {rawsave(&line[3]); return NOCOMM;}//save raw
	printf("\nERROR");//Command not found
	return NOCOMM;
}


//########################################################
void HelpScreen(void)
//########################################################
{
	puts("\nCommand List:\n");
	puts("Q         return to OS\n");
	puts("H or ?    show this help\n");
	puts("V         show version info\n");
	puts("I         drive informations\n");
	puts("D         list current directory\n");
	puts("M AAAA or (name) shows Memory or File\n");
	puts("J AAAA    jump to AAAA\n");
	puts("L (name)  load file (name) ^=~\n");
	puts("S AAAA EEEE SSSS (name)\n");
	puts("  save KCC file (name) SSSS=FFFF noauto\n");
	puts("T AAAA EEEE SSSS (name)\n");
	puts("  save TAP file (name) SSSS=FFFF noauto\n");
	puts("LR AAAA (name) raw load file\n");
	puts("SR AAAA EEEE (name) raw save file\n");
	puts("CD (name) change dir (name)\n");
	puts("MD (name) make dir (name)\n");
	puts("RM (name) remove (name)\n");
	puts("RN (name),(new name) rename (name)\n");
	puts("WB        WBASIC\n");
	puts("EOR       RAM-Ende\n");
	puts("EOB       BASIC-Ende\n");
	ser_putc(EOT);
}


//########################################################
void listdir(void)// das DIR-Commando...
//########################################################
{
	unsigned char z;z=0;
	if(Findfirst()!=0)
	{
		// printf("\n");
		do
		{
			if(ffblk.ff_attr==ATTR_FILE)
			{
				printf("\nF  %s  ",&ffblk.ff_name);
				printf("% 8lu",ffblk.ff_fsize);
			}
			if(ffblk.ff_attr==ATTR_DIRECTORY)
			{
				printf("\nD  %s",&ffblk.ff_name);
				printf("<DIR>");
			}
			z++;if(z==16){/*printf(">");*/ser_putc(0x80);if(!Command()){z=0;};}
			
		}while (Findnext()!=0 && z<16);////next oder Abbruch
	}
}
//########################################################
void listdirl(void)// listlongname...experimentell!
//########################################################
{
	unsigned char z;z=0;
	if(Findfirst()!=0)
	{
		do
		{
			if(ffblk.ff_attr==ATTR_FILE)
			{
				printf("\nF% 6lu ",ffblk.ff_fsize);
				printf("%s",&ffblk.ff_longname);
				if(ffblk.ff_longname[0]==0)printf("%s",&ffblk.ff_name);
			}
			if(ffblk.ff_attr==ATTR_DIRECTORY)
			{
				printf("\n<DIR>   ");
				printf("%s",&ffblk.ff_longname);
				if(ffblk.ff_longname[0]==0)printf("%s",&ffblk.ff_name);
			}
			z++;if(z==16){ser_putc(0x80);if(!Command()){z=0;};}

		}while (Findnext()!=0 && z<16);////next oder Abbruch
	}
}
//########################################################
unsigned int EOR(void) //RAM-Ende KC erfragen
//########################################################
{
	sendstring(7);
	unsigned int i;
	i= (getp()+((getp())<<8));// -100hex für das Programm(Überschreibschutz)
	if(FindName("load87")==FULL_MATCH)i-=0x100;//?...
	return (i);				 // braucht man bei Megarom nicht laeuft da im ROM!
}
//########################################################
unsigned int EOB(void) //BASIC-Ende KC erfragen
//########################################################
{
	sendstring(0);
	unsigned int i;
	i= (getp()+((getp())<<8));
	return (i);
}
//########################################################
void sendstring(unsigned int tab) //für längere Sequenzen zum KC
//########################################################
{
	unsigned char c;
	while((c=pgm_read_byte(&dat[tab++]))!=0xaa)putp(c);
}
